"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getElementModelFromTagName = getElementModelFromTagName;

var _HTMLElementModel = require("./HTMLElementModel");

const UA_ANCHOR_COL = '#245dc1';
const UA_GRAY = '#CCC';
const UA_MARGIN_HZ = 30;
const bigMarginTopBottomStyle = {
  marginTop: '1em',
  marginBottom: '1em'
};
const shortMarginTopBottomStyle = {
  marginTop: '.5em',
  marginBottom: '.5em'
};
const lineThroughStyle = {
  textDecorationLine: 'line-through'
};
const italicStyle = {
  fontStyle: 'italic'
};
const monoStyle = {
  fontFamily: 'monospace'
};
const boldStyle = {
  fontWeight: 'bold'
};
const spacedBlockStyle = { ...bigMarginTopBottomStyle,
  marginLeft: UA_MARGIN_HZ,
  marginRight: UA_MARGIN_HZ
};
const anchorStyle = {
  color: UA_ANCHOR_COL,
  textDecorationLine: 'underline',
  textDecorationColor: UA_ANCHOR_COL
};
const leftBorderQuoteStyle = {
  borderLeftWidth: 2,
  borderLeftColor: UA_GRAY
};
const dottedUnderlineStyle = {
  textDecorationLine: 'underline',
  textDecorationStyle: 'dotted'
};
const solidUnderlineStyle = {
  textDecorationLine: 'underline',
  textDecorationStyle: 'solid'
}; // TODO, support directional styles

const listStyles = { ...bigMarginTopBottomStyle,
  paddingLeft: 30
};
const preStyles = {
  whiteSpace: 'pre',
  ...monoStyle,
  ...bigMarginTopBottomStyle
};

function headerStyle(fontSize, marginSize) {
  return {
    fontSize,
    marginTop: marginSize,
    marginBottom: marginSize,
    ...boldStyle
  };
}

const sectioningModelMap = {
  address: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'address',
    mixedUAStyles: italicStyle
  }),
  article: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'article'
  }),
  aside: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'aside'
  }),
  body: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'body'
  }),
  footer: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'footer'
  }),
  h1: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'h1',
    mixedUAStyles: headerStyle('2em', '.67em')
  }),
  h2: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'h2',
    mixedUAStyles: headerStyle('1.5em', '.83em')
  }),
  h3: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'h3',
    mixedUAStyles: headerStyle('1.17em', '1em')
  }),
  h4: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'h4',
    mixedUAStyles: headerStyle('1em', '1.33em')
  }),
  h5: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'h5',
    mixedUAStyles: headerStyle('.83em', '1.67em')
  }),
  h6: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'h6',
    mixedUAStyles: headerStyle('.67em', '2.33em')
  }),
  header: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'header'
  }),
  hgroup: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'hgroup'
  }),
  nav: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'nav'
  }),
  section: new _HTMLElementModel.HTMLElementModel({
    category: 'sectioning',
    tagName: 'section'
  })
};
const unsupportedModelMap = {
  area: new _HTMLElementModel.HTMLElementModel({
    tagName: 'area',
    category: 'untranslatable',
    isVoid: true
  }),
  map: new _HTMLElementModel.HTMLElementModel({
    tagName: 'map',
    category: 'untranslatable'
  })
};
const attribsModelMap = {
  accesskey: new _HTMLElementModel.HTMLElementModel({
    tagName: 'accesskey',
    category: 'untranslatable'
  }),
  caption: new _HTMLElementModel.HTMLElementModel({
    tagName: 'caption',
    category: 'untranslatable'
  }),
  col: new _HTMLElementModel.HTMLElementModel({
    tagName: 'col',
    category: 'untranslatable',
    isVoid: true
  }),
  colgroup: new _HTMLElementModel.HTMLElementModel({
    tagName: 'colgroup',
    category: 'untranslatable'
  }),
  datalist: new _HTMLElementModel.HTMLElementModel({
    tagName: 'datalist',
    category: 'untranslatable'
  }),
  source: new _HTMLElementModel.HTMLElementModel({
    tagName: 'source',
    category: 'untranslatable',
    isVoid: true
  }),
  track: new _HTMLElementModel.HTMLElementModel({
    tagName: 'track',
    category: 'untranslatable',
    isVoid: true
  }),
  optgroup: new _HTMLElementModel.HTMLElementModel({
    tagName: 'optgroup',
    category: 'untranslatable'
  }),
  option: new _HTMLElementModel.HTMLElementModel({
    tagName: 'option',
    category: 'untranslatable'
  }),
  param: new _HTMLElementModel.HTMLElementModel({
    tagName: 'param',
    category: 'untranslatable',
    isVoid: true
  })
};
const interactiveModelMap = {
  button: new _HTMLElementModel.HTMLElementModel({
    tagName: 'button',
    category: 'interactive'
  }),
  fieldset: new _HTMLElementModel.HTMLElementModel({
    tagName: 'fieldset',
    category: 'interactive'
  }),
  form: new _HTMLElementModel.HTMLElementModel({
    tagName: 'form',
    category: 'interactive'
  }),
  input: new _HTMLElementModel.HTMLElementModel({
    tagName: 'input',
    category: 'interactive',
    isVoid: true
  }),
  label: new _HTMLElementModel.HTMLElementModel({
    tagName: 'label',
    category: 'interactive'
  }),
  legend: new _HTMLElementModel.HTMLElementModel({
    tagName: 'legend',
    category: 'interactive'
  }),
  meter: new _HTMLElementModel.HTMLElementModel({
    tagName: 'meter',
    category: 'interactive'
  }),
  progress: new _HTMLElementModel.HTMLElementModel({
    tagName: 'progress',
    category: 'interactive'
  }),
  select: new _HTMLElementModel.HTMLElementModel({
    tagName: 'select',
    category: 'interactive'
  }),
  details: new _HTMLElementModel.HTMLElementModel({
    tagName: 'details',
    category: 'interactive'
  }),
  dialog: new _HTMLElementModel.HTMLElementModel({
    tagName: 'dialog',
    category: 'interactive'
  }),
  output: new _HTMLElementModel.HTMLElementModel({
    tagName: 'output',
    category: 'interactive'
  }),
  summary: new _HTMLElementModel.HTMLElementModel({
    tagName: 'summary',
    category: 'interactive'
  }),
  textarea: new _HTMLElementModel.HTMLElementModel({
    tagName: 'textarea',
    category: 'interactive'
  })
};
const metadataModelMap = {
  base: new _HTMLElementModel.HTMLElementModel({
    tagName: 'base',
    category: 'untranslatable',
    isVoid: true
  }),
  head: new _HTMLElementModel.HTMLElementModel({
    tagName: 'head',
    category: 'untranslatable',
    isOpaque: true
  }),
  link: new _HTMLElementModel.HTMLElementModel({
    tagName: 'link',
    category: 'untranslatable',
    isVoid: true
  }),
  meta: new _HTMLElementModel.HTMLElementModel({
    tagName: 'meta',
    category: 'untranslatable',
    isVoid: true
  }),
  title: new _HTMLElementModel.HTMLElementModel({
    tagName: 'title',
    category: 'untranslatable'
  })
};
const untranslatableModelMap = { ...attribsModelMap,
  ...interactiveModelMap,
  ...unsupportedModelMap,
  ...metadataModelMap
};
const groupingModelMap = {
  blockquote: new _HTMLElementModel.HTMLElementModel({
    tagName: 'blockquote',
    category: 'grouping',
    getUADerivedStyleFromAttributes: attributes => {
      if (attributes.type === 'cite') {
        return leftBorderQuoteStyle;
      }

      return null;
    },
    mixedUAStyles: spacedBlockStyle
  }),
  dd: new _HTMLElementModel.HTMLElementModel({
    tagName: 'dd',
    category: 'grouping',
    mixedUAStyles: {
      marginLeft: UA_MARGIN_HZ
    }
  }),
  div: new _HTMLElementModel.HTMLElementModel({
    tagName: 'div',
    category: 'grouping'
  }),
  dl: new _HTMLElementModel.HTMLElementModel({
    tagName: 'dl',
    category: 'grouping',
    mixedUAStyles: bigMarginTopBottomStyle
  }),
  dt: new _HTMLElementModel.HTMLElementModel({
    tagName: 'dt',
    category: 'grouping',
    mixedUAStyles: boldStyle
  }),
  figcaption: new _HTMLElementModel.HTMLElementModel({
    tagName: 'figcaption',
    category: 'grouping',
    mixedUAStyles: { ...italicStyle,
      textAlign: 'center'
    }
  }),
  figure: new _HTMLElementModel.HTMLElementModel({
    tagName: 'figure',
    category: 'grouping',
    mixedUAStyles: spacedBlockStyle
  }),
  hr: new _HTMLElementModel.HTMLElementModel({
    tagName: 'hr',
    category: 'grouping',
    mixedUAStyles: {
      marginLeft: 'auto',
      marginRight: 'auto',
      height: 1,
      width: '100%',
      backgroundColor: UA_GRAY,
      ...shortMarginTopBottomStyle
    }
  }),
  li: new _HTMLElementModel.HTMLElementModel({
    tagName: 'li',
    category: 'grouping'
  }),
  main: new _HTMLElementModel.HTMLElementModel({
    tagName: 'main',
    category: 'grouping'
  }),
  menu: new _HTMLElementModel.HTMLElementModel({
    tagName: 'menu',
    category: 'grouping'
  }),
  ol: new _HTMLElementModel.HTMLElementModel({
    tagName: 'ol',
    category: 'grouping',
    mixedUAStyles: { ...listStyles,
      listStyleType: 'decimal'
    }
  }),
  p: new _HTMLElementModel.HTMLElementModel({
    tagName: 'p',
    category: 'grouping',
    mixedUAStyles: bigMarginTopBottomStyle
  }),
  pre: new _HTMLElementModel.HTMLElementModel({
    tagName: 'pre',
    category: 'grouping',
    mixedUAStyles: preStyles
  }),
  xmp: new _HTMLElementModel.HTMLElementModel({
    tagName: 'xmp',
    category: 'grouping',
    mixedUAStyles: preStyles
  }),
  listing: new _HTMLElementModel.HTMLElementModel({
    tagName: 'listing',
    category: 'grouping',
    mixedUAStyles: preStyles
  }),
  plaintext: new _HTMLElementModel.HTMLElementModel({
    tagName: 'plaintext',
    category: 'grouping',
    mixedUAStyles: preStyles
  }),
  ul: new _HTMLElementModel.HTMLElementModel({
    tagName: 'ul',
    category: 'grouping',
    mixedUAStyles: { ...listStyles,
      listStyleType: 'disc'
    }
  }),
  dir: new _HTMLElementModel.HTMLElementModel({
    tagName: 'dir',
    category: 'grouping',
    mixedUAStyles: { ...listStyles,
      listStyleType: 'disc'
    }
  })
};
const tabularModelMap = {
  table: new _HTMLElementModel.HTMLElementModel({
    tagName: 'table',
    category: 'tabular'
  }),
  tbody: new _HTMLElementModel.HTMLElementModel({
    tagName: 'tbody',
    category: 'tabular'
  }),
  td: new _HTMLElementModel.HTMLElementModel({
    tagName: 'td',
    category: 'tabular'
  }),
  tfoot: new _HTMLElementModel.HTMLElementModel({
    tagName: 'tfoot',
    category: 'tabular'
  }),
  th: new _HTMLElementModel.HTMLElementModel({
    tagName: 'th',
    category: 'tabular'
  }),
  thead: new _HTMLElementModel.HTMLElementModel({
    tagName: 'thead',
    category: 'tabular'
  }),
  tr: new _HTMLElementModel.HTMLElementModel({
    tagName: 'tr',
    category: 'tabular'
  })
}; // Embedded elements are considered "opaque", i.e. no children are meant to be
// translated. A reference to domChildren will be available on the rendering
// end.

const embeddedModelMap = {
  audio: new _HTMLElementModel.HTMLElementModel({
    tagName: 'audio',
    category: 'embedded',
    isVoid: false // allows tracks

  }),
  canvas: new _HTMLElementModel.HTMLElementModel({
    tagName: 'canvas',
    category: 'embedded',
    isVoid: false // allows specific content

  }),
  embed: new _HTMLElementModel.HTMLElementModel({
    tagName: 'embed',
    category: 'embedded',
    isVoid: true
  }),
  iframe: new _HTMLElementModel.HTMLElementModel({
    tagName: 'iframe',
    category: 'embedded',
    isVoid: true
  }),
  img: new _HTMLElementModel.HTMLElementModel({
    tagName: 'img',
    category: 'embedded',
    isVoid: true
  }),
  math: new _HTMLElementModel.HTMLElementModel({
    tagName: 'math',
    category: 'embedded',
    isVoid: false // allows mathml elems

  }),
  object: new _HTMLElementModel.HTMLElementModel({
    tagName: 'object',
    category: 'embedded',
    isVoid: false // allows params

  }),
  picture: new _HTMLElementModel.HTMLElementModel({
    tagName: 'picture',
    category: 'embedded',
    isVoid: false // allows source and img

  }),
  svg: new _HTMLElementModel.HTMLElementModel({
    tagName: 'svg',
    category: 'embedded',
    isVoid: false // allows svg elems

  }),
  video: new _HTMLElementModel.HTMLElementModel({
    tagName: 'video',
    category: 'embedded',
    isVoid: false // allows source, tracks + transparent

  })
};
const editsModelMap = {
  ins: new _HTMLElementModel.HTMLElementModel({
    tagName: 'ins',
    category: 'edits',
    mixedUAStyles: solidUnderlineStyle
  }),
  del: new _HTMLElementModel.HTMLElementModel({
    tagName: 'del',
    category: 'edits',
    mixedUAStyles: lineThroughStyle
  })
};
const textLevelModelMap = {
  em: new _HTMLElementModel.HTMLElementModel({
    tagName: 'em',
    category: 'textual',
    mixedUAStyles: italicStyle
  }),
  strong: new _HTMLElementModel.HTMLElementModel({
    tagName: 'strong',
    category: 'textual',
    mixedUAStyles: boldStyle
  }),
  small: new _HTMLElementModel.HTMLElementModel({
    tagName: 'small',
    category: 'textual',
    mixedUAStyles: {
      fontSize: 'smaller'
    }
  }),
  big: new _HTMLElementModel.HTMLElementModel({
    tagName: 'big',
    category: 'textual',
    mixedUAStyles: {
      fontSize: 'larger'
    }
  }),
  s: new _HTMLElementModel.HTMLElementModel({
    tagName: 's',
    category: 'textual',
    mixedUAStyles: lineThroughStyle
  }),
  strike: new _HTMLElementModel.HTMLElementModel({
    tagName: 'strike',
    category: 'textual',
    mixedUAStyles: lineThroughStyle
  }),
  cite: new _HTMLElementModel.HTMLElementModel({
    tagName: 'cite',
    category: 'textual',
    mixedUAStyles: italicStyle
  }),
  q: new _HTMLElementModel.HTMLElementModel({
    tagName: 'q',
    category: 'textual' // default style, content: "open,close-quote"

  }),
  dfn: new _HTMLElementModel.HTMLElementModel({
    tagName: 'dfn',
    category: 'textual',
    mixedUAStyles: italicStyle
  }),
  abbr: new _HTMLElementModel.HTMLElementModel({
    tagName: 'abbr',
    category: 'textual',
    mixedUAStyles: dottedUnderlineStyle
  }),
  acronym: new _HTMLElementModel.HTMLElementModel({
    tagName: 'acronym',
    category: 'textual',
    mixedUAStyles: dottedUnderlineStyle
  }),
  ruby: new _HTMLElementModel.HTMLElementModel({
    tagName: 'ruby',
    category: 'textual'
  }),
  rt: new _HTMLElementModel.HTMLElementModel({
    tagName: 'rt',
    category: 'textual'
  }),
  rp: new _HTMLElementModel.HTMLElementModel({
    tagName: 'rp',
    category: 'textual'
  }),
  data: new _HTMLElementModel.HTMLElementModel({
    tagName: 'data',
    category: 'textual'
  }),
  time: new _HTMLElementModel.HTMLElementModel({
    tagName: 'time',
    category: 'textual'
  }),
  code: new _HTMLElementModel.HTMLElementModel({
    tagName: 'code',
    category: 'textual',
    mixedUAStyles: monoStyle
  }),
  tt: new _HTMLElementModel.HTMLElementModel({
    tagName: 'tt',
    category: 'textual',
    mixedUAStyles: monoStyle
  }),
  var: new _HTMLElementModel.HTMLElementModel({
    tagName: 'var',
    category: 'textual',
    mixedUAStyles: italicStyle
  }),
  samp: new _HTMLElementModel.HTMLElementModel({
    tagName: 'samp',
    category: 'textual',
    mixedUAStyles: monoStyle
  }),
  kbd: new _HTMLElementModel.HTMLElementModel({
    tagName: 'kbd',
    category: 'textual',
    mixedUAStyles: monoStyle
  }),
  sub: new _HTMLElementModel.HTMLElementModel({
    tagName: 'sub',
    category: 'textual',
    mixedUAStyles: {
      textAlignVertical: 'bottom',
      fontSize: 'smaller'
    }
  }),
  sup: new _HTMLElementModel.HTMLElementModel({
    tagName: 'sup',
    category: 'textual',
    mixedUAStyles: {
      textAlignVertical: 'top',
      fontSize: 'smaller'
    }
  }),
  i: new _HTMLElementModel.HTMLElementModel({
    tagName: 'i',
    category: 'textual',
    mixedUAStyles: italicStyle
  }),
  b: new _HTMLElementModel.HTMLElementModel({
    tagName: 'b',
    category: 'textual'
  }),
  u: new _HTMLElementModel.HTMLElementModel({
    tagName: 'u',
    category: 'textual',
    mixedUAStyles: solidUnderlineStyle
  }),
  mark: new _HTMLElementModel.HTMLElementModel({
    tagName: 'mark',
    category: 'textual',
    mixedUAStyles: {
      backgroundColor: 'yellow',
      color: 'black'
    }
  }),
  bdi: new _HTMLElementModel.HTMLElementModel({
    tagName: 'bdi',
    category: 'textual' // unicode-bidi: isolate;

  }),
  bdo: new _HTMLElementModel.HTMLElementModel({
    tagName: 'bdo',
    category: 'textual' //  unicode-bidi: isolate-override;

  }),
  span: new _HTMLElementModel.HTMLElementModel({
    tagName: 'span',
    category: 'textual'
  }),
  br: new _HTMLElementModel.HTMLElementModel({
    tagName: 'br',
    category: 'textual',
    isVoid: true
  }),
  wbr: new _HTMLElementModel.HTMLElementModel({
    tagName: 'wbr',
    category: 'textual',
    isVoid: true
  })
};
const elementsModelMap = {
  a: new _HTMLElementModel.HTMLElementModel({
    tagName: 'a',
    category: 'anchor',
    getUADerivedStyleFromAttributes: attributes => {
      if (typeof attributes.href === 'string') {
        return anchorStyle;
      }

      return null;
    }
  }),
  ...textLevelModelMap,
  ...editsModelMap,
  ...embeddedModelMap,
  ...tabularModelMap,
  ...groupingModelMap,
  ...sectioningModelMap,
  ...interactiveModelMap,
  ...untranslatableModelMap
};

function getElementModelFromTagName(tagName) {
  if (Object.prototype.hasOwnProperty.call(elementsModelMap, tagName)) {
    return elementsModelMap[tagName];
  }

  return new _HTMLElementModel.HTMLElementModel({
    tagName,
    category: 'custom'
  });
}
//# sourceMappingURL=elements-model.js.map